from collections import Counter
import re

import pandas as pd
import networkx as nx

from ..api.nb_search_api import get_df, get_konks
from ..text.nbtokenizer import tokenize
from ..nbtext import frame, frame_sort

def phrase_plots(
    phrase_sets,
    title='aftenposten',
    fra = 1960,
    til = 2020,
    step=5,
    rot=0,
    colours = ['r', 'b','g','y','m','c']
):
    df_all = []
    for f in phrase_sets:
        df_all.append(frame(get_df(f, title= title), ', '.join(f)))
    df = pd.concat(df_all, sort=False)
    df.index = df.index.astype(int)
    df = df.sort_index()
    df['bins'] = pd.cut(df.index, range(fra, til, step), precision=0)
    df.groupby('bins').sum().plot(kind='bar', color=colours, figsize=(15,5), rot=rot)
    return

def phrase_plots_anno(
    phrase_sets,
    title = 'aftenposten',
    fra = 1960,
    til = 2020,
    rot=0,
    colours = ['r', 'b','g']
):
    df_all = []
    for f in phrase_sets:
        df_all.append(frame(get_df(f, title = title), ', '.join(f)))
    df = pd.concat(df_all, sort = False)
    df.index = df.index.astype(int)
    df = df.sort_index()
    #df['bins'] = pd.cut(df.index, range(fra, til, step), precision=0)
    df.plot(kind = 'bar', figsize = (15,5), rot = rot, color = colours)
    return

def graph_from_df(df, threshold = 100):
    edges =  []
    normalizer = {(x, y): df.stack()[(x,x)]*df.stack()[(y,y)] for (x,y) in df.stack().index}
    for (x, y) in df.stack().index:
        if x != y:
            if df.stack()[(x,y)] > threshold:
                edges.append([x,y,df.stack()[(x,y)]/normalizer[(x,y)]])
    G = nx.Graph()
    G.add_weighted_edges_from(edges)
    return G


def get_all_konks(term, urns):
    konks = []
    for u in urns:
        konks += get_konks(u, term)
    return konks

def collocations_from_nb(word, corpus):
    """Get a concordance, and count the words in it. 
    Assume konks reside a dataframe with columns 'after' and 'before'"""
    concordance = frame(get_all_konks(word, corpus))
    return frame_sort(frame(Counter(tokenize(' '.join(concordance['after'].values + concordance['before'].values))), word))

def count_from_conc(concordance):
    """From a concordance, count the words in it. 
    Assume konks reside a dataframe with columns 'after' and 'before'"""
    word = concordance['word'][0]
    return frame_sort(
        frame(
            Counter(
                tokenize(' '.join(concordance['after'].values + concordance['before'].values))
            ), 
            word
        )
    )



from collections import Counter

import matplotlib.pyplot as plt
import networkx as nx
import numpy as np
import requests
import seaborn as sns
from community import community_louvain
from matplotlib import colors as mcolors
from matplotlib.pylab import rcParams
from networkx.algorithms.community import k_clique_communities

from dhlab.nbtext import urn_coll, urn_coll_words, frame, get_freq, make_graph_from_result

colors = dict(mcolors.BASE_COLORS, **mcolors.CSS4_COLORS)
rcParams['figure.figsize'] = 15, 10


def cutdown(x):
    return x.subgraph([n[0] for n in x.degree() if n[1] > 1])


def make_graph_corp(word, corpus='eng'):
    query = (
        f"http://www.nb.no/sp_tjenester/beta/ngram_1/galaxies/query"
        f"?terms={word}&lang=all&corpus={corpus}"
    )
    result = requests.get(query)
    return make_graph_from_result(result)


def make_graph(word):
    query = f"http://www.nb.no/sp_tjenester/beta/ngram_1/galaxies/query?terms={word}"
    result = requests.get(query)
    return make_graph_from_result(result)


def draw_graph(G, nodelist: list = None, h=15, v=10, fontsize=12, layout=nx.spring_layout,
               arrows=False, node_color='orange', node_size=100,
               font_color='black'):
    if nodelist is not None:
        G = G.subgraph(nodelist)
    x, y = rcParams['figure.figsize']
    rcParams['figure.figsize'] = h, v
    pos = layout(G)
    ax = plt.subplot()
    ax.set_xticks([])
    ax.set_yticks([])
    nx.draw_networkx_labels(G, pos, font_size=fontsize, font_color=font_color)
    nx.draw_networkx_nodes(G, pos, alpha=0.1, node_color=node_color,
                           node_size=node_size)
    nx.draw_networkx_edges(G, pos, alpha=0.7, arrows=arrows,
                           edge_color='lightblue')

    rcParams['figure.figsize'] = x, y


def draw_graph_centrality(G, h=15, v=10, deltax=0, deltay=0, fontsize=18, k=0.2,
                          arrows=False, node_alpha=0.3, l_alpha=1,
                          node_color='blue', centrality=nx.degree_centrality,
                          font_color='black', threshold=0.01, multi=3000):
    node_dict = centrality(G)
    subnodes = dict(
        {x: node_dict[x] for x in node_dict if node_dict[x] >= threshold})
    x, y = rcParams['figure.figsize']
    rcParams['figure.figsize'] = h, v

    ax = plt.subplot()
    ax.set_xticks([])
    ax.set_yticks([])
    G = G.subgraph(subnodes)
    pos = nx.spring_layout(G, k=k)
    labelpos = {k: (v[0] + deltax, v[1] + deltay) for k, v in pos.items()}
    # print(labelpos)
    # print(pos)
    if l_alpha <= 1:
        nx.draw_networkx_labels(G, labelpos, font_size=fontsize, alpha=l_alpha,
                                font_color=font_color)
    nx.draw_networkx_nodes(G, pos, alpha=node_alpha,
                           node_color=range(len(subnodes.keys())),
                           # Cannot find reference 'Blues' in 'cm.py'
                           cmap=plt.cm.Blues, nodelist=subnodes.keys(),
                           node_size=[v * multi for v in subnodes.values()])
    nx.draw_networkx_edges(G, pos, alpha=0.4, arrows=arrows,
                           edge_color='lightblue')

    rcParams['figure.figsize'] = x, y


def draw_graph_centrality2(G, Subsets=None, h=15, v=10, deltax=0, deltay=0,
                           fontsize=18, k=0.2, arrows=False,
                           node_alpha=0.3, l_alpha=1, node_color='blue',
                           centrality=nx.degree_centrality,
                           font_color='black',
                           threshold=0.01,
                           multi=3000,
                           edge_color='olive',
                           edge_alpha=0.1,
                           colstart=0.2,
                           coldark=0.5):
    if Subsets is None:
        Subsets = []
    # W0621: Redefining name 'colors' from outer scope (line 16) (redefined-outer-name)
    colors = dict(mcolors.BASE_COLORS, **mcolors.CSS4_COLORS)  # W0612: Unused variable 'colors'
    node_dict = centrality(G)
    subnodes = {x: node_dict[x] for x in node_dict if node_dict[x] >= threshold}
    x, y = rcParams['figure.figsize']
    rcParams['figure.figsize'] = h, v

    ax = plt.subplot()
    ax.set_xticks([])
    ax.set_yticks([])
    # G = G.subgraph(subnodes)
    glob_col = sns.hls_palette(len(G), h=colstart, l=coldark)[0]
    pos = nx.spring_layout(G, k=k)
    labelpos = {k: (v[0] + deltax, v[1] + deltay) for k, v in pos.items()}
    # print(labelpos)
    # print(pos)
    if l_alpha <= 1:
        nx.draw_networkx_labels(G, labelpos, font_size=fontsize, alpha=l_alpha,
                                font_color=font_color)
    sub_color = 0  # W0612: Unused variable 'sub_color' (unused-variable)
    if Subsets != []:
        i = 0
        colpalette = sns.hls_palette(len(Subsets), h=colstart, l=coldark)
        # print(colpalette)
        for Sub in Subsets:
            sublist = dict({x: subnodes[x] for x in subnodes if x in Sub})
            # print(sublist)
            # sub_col = list(colors.values())[np.random.randint(20,100)]
            sub_col = colpalette[i]
            # print(i, sub_col, sublist.keys())
            # print(i, sub_col)
            nx.draw_networkx_nodes(G, pos, alpha=node_alpha,
                                   node_color=[sub_col],
                                   nodelist=list(sublist.keys()),
                                   node_size=[v * multi for v in
                                              sublist.values()])
            i += 1
    else:
        nx.draw_networkx_nodes(G, pos, alpha=node_alpha, node_color=glob_col,
                               nodelist=subnodes.keys(),
                               node_size=[v * multi for v in subnodes.values()])

    nx.draw_networkx_edges(G, pos, alpha=edge_alpha, arrows=arrows,
                           edge_color=edge_color)

    rcParams['figure.figsize'] = x, y


# Set palette using: sns.hls_palette(10, h=.6, l=.1)


def sentrale(Graph, top=20):
    # mc = Counter([('ord',0)])
    # SubGraph = nx.Graph()
    # SubGraph.add_edges_from([(x,y) for (x,y) in Graph.edges() \
    #   if Graph.degree(x)>1 and Graph.degree(y)>1])
    # if Graph.__len__() > 0:
    mc = Counter(nx.closeness_centrality(Graph)).most_common(top)
    return mc


def mcommunity(Graph, random=10):
    G = Graph.to_undirected()

    m_partition = community_louvain.best_partition(G, random_state=random)
    # print(m_partition)
    list_nodes = []
    for com in set(m_partition.values()):
        list_nodes += [
            {nodes for nodes in m_partition.keys() if m_partition[nodes] == com}
        ]
    return list_nodes


def kcliques(agraph):
    i = 3
    x = list(k_clique_communities(agraph, i))
    comms = {}
    while x and isinstance(x, list):
        # print(x)
        j = 1
        for el in x:
            comms[(i, j)] = el
            j += 1
        i += 1
        x = list(k_clique_communities(agraph, i))
    return comms


def subsetgraph(comms, centrals, labels=2):
    """comms is communities """
    subgraph = nx.DiGraph()
    comkeys = sorted(comms.keys(), key=lambda x: x[0], reverse=True)
    for i, top in enumerate(comkeys):
        label_small = str(top[0]) + str(top[1])
        small_ordered = Counter(
            {r: centrals[r] for r in comms[top]}).most_common(labels)
        label_small = label_small + ' ' + ' '.join(
            [x[0] for x in small_ordered])
        subgraph.add_node(label_small)
        j = i + 1
        found = False
        while j < len(comkeys) and not found:
            nodej = comms[comkeys[j]]

            found = comms[top].issubset(nodej)
            # print(top,comkeys[j], found)
            if found:
                label_large = str(comkeys[j][0]) + str(comkeys[j][1])
                large_ordered = Counter(
                    {r: centrals[r] for r in nodej}).most_common(labels)
                label_large = label_large + ' ' + ' '.join(
                    [x[0] for x in large_ordered])
                # print(label_small, label_large)
                subgraph.add_edge(label_small, label_large)
            j += 1
    return subgraph


def make_cliques(words, lable_num=2):
    ggg = make_graph(words).to_undirected()
    centrals = nx.closeness_centrality(ggg)
    coms = kcliques(ggg)
    sg = subsetgraph(coms, centrals, lable_num)
    return (ggg, coms, sg)


def make_w_graph(weight_matrix):
    """weight_matrix a list on the form [((x,y,weight), ...]"""

    G = nx.Graph()
    G.add_weighted_edges_from(weight_matrix)
    return G


def make_cliques_from_graph(G, lable_num=2):
    ggg = G
    centrals = nx.closeness_centrality(ggg)
    coms = kcliques(ggg)
    sg = subsetgraph(coms, centrals, lable_num)
    return (ggg, coms, sg)


def my_layout(G):
    """For grafer fra make_cliques der koden ligger i de to første tallene"""
    pos = {}
    for i in G.nodes():
        x = i.split()[0]
        pos[i] = (int(x[0]), int(x[1]))
    return pos


'''
def tree_layout(G):
    """For grafer fra make_cliques der koden ligger i de to første tallene.

    OBS! Denne funksjonen vil ikke fungere uten refaktorering.
    Se kommentarer på de aktuelle linjene som må fikses.
    """
    pos = {}
    roots = root_nodes(G)  # W0612: Unused variable 'roots' (unused-variable)
    for r in G.nodes():  # W0612: Unused variable 'r' (unused-variable)
        x = i.split()[0]  # E0602: Undefined variable 'i' (undefined-variable)
        pos[i] = (int(x[0]), int(x[1]))
    return pos
'''


def root_nodes(G):
    res = []
    for x in G.nodes():
        found = False
        for y in G.nodes():
            found = found or (x, y) in G.edges()
            if found:
                break
        if not found:
            res += [x]
    return res


def tree_positions(Tree, spacing, increment=1):
    root = root_nodes(Tree)[0]
    return tree_pos(root, Tree, 1, spacing, 3, 3, level_increment=increment)[0]


def tree_pos(x, G, level, spacing, num, left_edge, level_increment=1):
    """Draw from left to right for left_edge"""
    positions = {}
    daughters = [y for (y, z) in G.edges() if x == z]
    if daughters == []:
        positions[x] = (left_edge, level)
        d_left = left_edge
    else:
        i = 1
        d_left = left_edge
        vals = []
        d_level = level + level_increment
        for d in daughters:
            d_positions, d_width = tree_pos(
                d,
                G,
                d_level - np.random.randint(1, 5) * level_increment / 10,
                spacing,
                i,
                d_left,
                level_increment=level_increment)
            i += 1
            d_left += spacing + d_width
            positions.update(d_positions)
            vals += [d_positions[d][0]]
            # print(vals)
        averagex = np.mean(vals)
        positions[x] = (averagex, level)
    return positions, d_left


def forest(G):
    roots = root_nodes(G)
    g_forest = []
    for r in roots:
        nr = node_set(r, G)
        g_forest += [G.subgraph(nr)]
    return g_forest


def node_set(root, G):
    span = [root]
    daughters = [y for (y, z) in G.edges() if z == root]
    if daughters != []:
        for d in daughters:
            span += node_set(d, G)
    return span


def draw_tree(G, node_size=1, node_color='slategrey', n=2, m=1, h=10, v=10):
    # plt.subplot()
    draw_graph(G, h=h, v=v, layout=lambda g: tree_positions(g, n, increment=m),
               node_color=node_color, node_size=node_size, fontsize=18,
               arrows=False)
    fmin, fmax = plt.xlim()
    plt.xlim(fmin - 10, fmax + 10)
    # ax.set_xticks([])
    # ax.set_yticks([])
    # plt.savefig('krig.svg')


'''
def draw_forest(F, spacing, h=15, v=10, save_name=False):
    """Denne funksjonen vil ikke fungere uten refaktorering.

    Se kommentarer på/over de aktuelle linjene.
    """
    # rows = len(F)
    # row = 1
    # plt.figsize=(15,10)
    for tree in F:
        # print(tree.nodes())
        # plt.subplot(rows,row,1)
        # plt.figure(row)
        # row += 1

        # Expected type 'int', got 'float' instead for node_size
        draw_tree(tree, node_size=0.5, h=h, v=v)
        if save_name:
            # E0602: Undefined variable 'row' (undefined-variable)
            plt.savefig(f'{save_name}-{row}.png', dpi=300)
'''

def print_list_of_sets(los):
    for x in los:
        print(', '.join(x), '\n')


def print_sets(graph):
    for x in graph[1]:
        print(x, ', '.join(graph[1][x]), '\n')
    return True


def make_collocation_graph(target, top=15, urns=None, cutoff=0, cut_val=2,
                           before=4, after=4, limit=1000):
    """Make a cascaded network from collocations"""
    if urns is None:
        urns = []
    antall = Counter()
    for urn in urns:
        antall += get_freq(urn[0], top=0, cutoff=0)

    korpus_totalen = frame(antall, 'total')
    Total = korpus_totalen[korpus_totalen > cut_val]

    if isinstance(target, str):
        target = target.split()

    I = urn_coll_words(target, urns=urns, before=before, after=after,
                       limit=limit)
    toppis = frame(I[0] ** 1.2 / Total['total'], target[0]).sort_values(
        by=target[0], ascending=False)

    # toppis[:top].index

    isgraf = {}
    for word in toppis[:top].index:
        if word.isalpha():
            isgraf[word] = urn_coll(word, urns=urns, before=before, after=after)

    isframe = {}
    for w, value in isgraf.items():
        isframe[w] = frame(value, w)

    tops = {}
    if len(target) == 1:
        tops[target[0]] = toppis
    else:
        tops['_'.join(target[:2])] = toppis
    for w, value in isframe.items():
        tops[w] = frame(value[w] ** 1.2 / Total['total'], w).sort_values(
            by=w, ascending=False)

    edges = []
    for w, value in tops.items():
        edges += [(w, coll) for coll in value[:top].index if coll.isalpha()]

    ice = nx.Graph()

    ice.add_edges_from(edges)

    return ice


def show_graph(G, spread=0.2, fontsize=10, deltax=0, deltay=0):
    return draw_graph_centrality2(G, mcommunity(G), k=spread, fontsize=fontsize,
                                  deltax=deltax, deltay=deltay)


def show_cliques(G):
    C = make_cliques_from_graph(G.to_undirected())
    for t in C[1]:
        print(t, ', '.join(C[1][t]))
        print()


def show_community(G):
    MC = mcommunity(G)
    for i, element in enumerate(MC):
        print(i + 1, ', '.join(element))
        print()
    return True


def community_dict(G):
    sorter = Counter(dict(nx.degree(G)))
    cd = {}
    for c in mcommunity(G):
        l = [(x, sorter[x]) for x in c if sorter[x] > 0]
        # print(l)
        l.sort(key=lambda i: i[1], reverse=True)
        # print(l)
        cd['-'.join([x[0] for x in l[:2]])] = [x[0] for x in l]
    return cd


def show_communities(G):
    Gc = community_dict(G)
    for c, value in Gc.items():
        print(c, ': ', ', '.join(value))
        print()


def reduce_MxM_graph(G, words, factor=0.01):
    Gm = nx.Graph()
    edges = []
    for x in G.edges(data=True):
        w = x[2]['weight']
        w1 = x[0]
        w2 = x[1]
        new_weight = w / (int(words.loc[w1]) * int(words.loc[w2]))
        if new_weight > factor:
            edges.append((w1, w2, new_weight))
    Gm.add_weighted_edges_from(edges)
    return Gm



import requests
import os
from IPython.display import HTML, Markdown, display

def printmd(S):
    display(Markdown(S))
    return

def download_from_github(file = None, user = None, repository = None, branch = 'master', overwrite = False, silent = False):
    """Fetch a file from Github and write it to working directory, 
    supply user and repository, 
    branch defaults to master
    overwrite: defaults to no overwrite
    silent: default is not silent"""
    
    nba = requests.get(
        f"https://raw.githubusercontent.com/{user}/{repository}/{branch}/{file}",
        headers={'Cache-Control': 'no-cache'}
        )
    if nba.status_code == 200:
        filename = f'{file}'
        file_exists = os.path.exists(filename)
        if file_exists and not(overwrite):
            if not silent:
                printmd(f"File {os.path.abspath(filename)} exists - call `download('{file}', overwrite = True)` in order to download module `{file}` anyway")
        else:
            nba = nba.text
            with open(filename,'w', encoding='UTF-8') as pyfile:
                pyfile.write(nba)
                pyfile.flush()
                pyfile.close()
            if not silent:
                printmd(f"Downloaded file `{os.path.abspath(file)}`")
    else:
        printmd(f"""Failed to download {file} with http code {nba.status_code}""")
        
    return

def get_file_from_github(url, overwrite = False, silent = False):
    """fetch a file on github, 
    it is enough with reference 
    it will look in raw user content for the file
    
    overwrite: defaults to no overwrite
    silent: default is not silent"""
    
    if url.startswith("https://github.com/") or url.startswith("github.com"):
        trail = url.split("github.com")[-1].replace('blob/', '')
        fileref = f"""https://raw.githubusercontent.com{trail}"""
    elif url.startswith("raw.githubusercontent"):
        fileref = f"""https://{url}"""
    elif url.startswith("https://raw.githubusercontent"):
        fileref = url
    else:
        return "is this is a file on github?"
        
    nba = requests.get(
        f"{fileref}",
        headers={'Cache-Control': 'no-cache'}
        )
    if nba.status_code == 200:
        filename = os.path.basename(fileref)
        file_exists = os.path.exists(filename)
        if file_exists and not(overwrite):
            if not silent:
                printmd(f"File {os.path.abspath(filename)} exists - call `download('{filename}', overwrite = True)` in order to download `{filename}` anyway")
        else:
            nba = nba.text
            with open(filename,'w', encoding='UTF-8') as pyfile:
                pyfile.write(nba)
                pyfile.flush()
                pyfile.close()
            if not silent:
                printmd(f"Downloaded file `{os.path.abspath(filename)}`")
    else:
        printmd(f"""Failed to download {fileref} with http response code {nba.status_code}""")
        
    return



import pandas as pd

from ..api.dhlab_api import document_corpus

class Corpus():
    def __init__(
        self,
        doctype = None,
        author = None,
        freetext = None,
        from_year = None,
        to_year = None,
        from_timestamp = None,
        to_timestamp = None,
        title = None,
        ddk = None,
        subject = None,
        lang = None,
        limit = None):
        
        self.corpus = document_corpus(
            doctype,
            author,
            freetext,
            from_year,
            to_year,
            from_timestamp,
            to_timestamp,
            title,
            ddk,
            subject,
            lang,
            limit
        )
        
        self.size = len(self.corpus)
        
        return


import ast
import os
import sys
from collections import Counter

import pandas as pd
import requests

from dhlab.nbtext import (
    make_network_name_graph,
    names,
    token_map,
    pure_urn,
    metadata
)


def names_from_corpus(korpus):
    """Find names in a larger corpus korpus is a frame with a column urn, or a list of urns """

    # urner = list(korpus['urn'])
    urner = pure_urn(korpus)
    alle_navn = combine_names(corpus_names(urner))
    return alle_navn


def count_names_corpus(korpus, token_map_):
    """Count names in a corpus using a token map,
    which groups different name tokens into one token.
    """

    res = {}
    urner = pure_urn(korpus)
    for urn in urner:
        try:
            res[urn] = count_name_strings(str(urn), token_map_).to_dict()[0]
        except:
            try:
                print('feil med:',
                      ', '.join([str(x) for x in metadata(str(urn))[0]]),
                      sys.exc_info()[0])
            except:
                print('Kunne ikke hente data for:', urn)
    return pd.DataFrame(
        pd.DataFrame(res).sum(axis=1).sort_values(ascending=False))


def names_from_excel(excelfil):
    """Get an edited Excel file, with names in Excel column A and values in column B"""

    navn = pd.read_excel(excelfil, index_col=0)
    navn.index = navn.index.dropna()
    navn = navn.to_dict()[0]
    navnedata = ({},) * 4

    for x in navn:
        try:
            xval = ast.literal_eval(x)
            navnedata[len(xval) - 1][xval] = navn[x]
        except:
            navnedata[0][x] = navn[x]
    return navnedata


def token_map_names(tmap):
    return [
               [z[0][0] for z in (tmap) if len(z[0]) == 1]
           ] + [
               [z[0] for z in (tmap) if len(z[0]) == 2]
           ] + [
               [z[0] for z in (tmap) if len(z[0]) == 3]
           ] + [
               [z[0] for z in (tmap) if len(z[0]) == 4]
           ]


# create a character network with only tokens in tokenmap
# see nb.make_network_name_graph in nbtext

def check_exit_conditions(filename):
    if filename == '':
        raise ValueError('Angi et filnavn')
    if os.path.exists(filename):
        raise FileExistsError(f'Filen {filename} eksisterer - prøv et nytt filnavn')


def names_to_token_map_file(wp, filename='', orient='column'):
    """Save token map to file for editing, exit if file exists"""

    # check  exit conditions
    try:
        check_exit_conditions(filename)
    except (ValueError, FileExistsError) as error:
        print(error)
        return

    # if all ok go ahead

    table_names = {}
    # print(wp)
    tmap = token_map(wp)
    # print(tmap)
    for (name, target) in tmap:
        x_str = ' '.join(target)
        y_str = ' '.join(name)
        table_names.setdefault(x_str, []).append(y_str)

    dfs = [pd.DataFrame({key: value}) for key, value in table_names.items()]

    df = pd.concat(dfs, axis=1)
    if orient == 'row':
        df = df.transpose()
    rv = True
    if filename.endswith('csv'):
        df.to_csv(filename)
    elif filename.endswith('xls'):
        df.to_excel(filename, index=orient == "row")
    else:
        rv = df
    return rv


def read_token_map_file(filename, sep=', ', orient='column'):
    """Read a token map from file, either xls or csv"""

    if filename.endswith('xls'):
        res = pd.read_excel(filename, index_col=0).dropna(how='all').fillna('')
    elif filename.endswith('csv'):
        res = pd.read_csv(filename, sep=sep, index_col=0).dropna(
            how='all').fillna('')
    if orient.startswith('row'):
        res = res.transpose()
    result = []
    for x in res:
        xt = tuple(x.split())
        for value in res[x]:
            vt = tuple(value.split())
            if vt != ():
                result.append((vt, xt))
    return result


def show_names(wp):
    """Display found names with frequency"""
    i = 1
    for c in wp:
        print('Lag', i)
        print("=========")
        print()
        i += 1
        for x in c.most_common():
            if isinstance(x[0], str):
                print("   ", x[0] + ' - ' + str(x[1]))
            else:
                print("   ", ' '.join(x[0]) + ' - ' + str(x[1]))
        print()


def character_network(urn, token_map_, names_=None):
    if names_ is None:
        names_ = token_map_names(token_map_)
    return make_network_name_graph(urn, names_, tokenmap=token_map_)


def count_name_strings(urn, token_map_, names_=None):
    """ return a count of the names in tokenmap"""
    if names_ is None:
        names_ = token_map_names(token_map_)

    if isinstance(urn, list):
        urn = urn[0]

    # tokens should be a list of list of tokens. If it is list of dicts pull out the keys (= tokens)
    # if isinstance(tokens[0], dict):
    #    tokens = [list(x.keys()) for x in tokens]

    res = requests.post("https://api.nb.no/ngram/word_counts",
                        json={'urn': urn, 'tokens': names_,
                              'tokenmap': token_map_})
    # print(r.text)

    return pd.read_json(res.json()).sort_values(by=0, ascending=False)


def corpus_names(corpus, ratio=0.5, cutoff=10):
    # check status of corpus if it is a frame or a list of list
    # for now assume it is a list of URNs
    urn_names = {}
    for urn in corpus:
        try:
            urn_names[urn] = names(urn, ratio=ratio, cutoff=cutoff)
        except:
            print("Fikk ikke laget navn for:", urn)
    return urn_names


def combine_names(namedict):
    total_names = [Counter()] * 4
    for urn in namedict:
        for i in range(4):
            total_names[i] += namedict[urn][i]
    return total_names


def filter_names(tm_names, gazetteers):
    """Filter name findings using a gazetteer
    - the gazetteer should consist of a list of first and last names.
    """

    # check single names
    def member(w, gazetteer):
        res = False
        if w in gazetteer:
            res = True
        elif w[-1] == 's' and w[:-1] in gazetteer:
            res = True
        return res

    def add_name(name_struct, struct, val):
        size = len(name_struct)
        if 0 < size <= 4:
            size -= 1
            # print(size, name_struct)
            # print(struct)
            if size == 0:
                # print(name_struct[0])
                name_struct = name_struct[0]
            if name_struct in struct[size]:
                struct[size][name_struct] += val
            else:
                struct[size][name_struct] = val
        return struct

    name_structure = [Counter(), Counter(), Counter(), Counter()]
    single_names = Counter()
    single_remove = Counter()
    for w in tm_names[0]:
        if member(w, gazetteers):
            single_names[w] = tm_names[0][w]
        else:
            single_remove[w] = tm_names[0][w]

    name_structure[0].update(single_names)

    # check double names (check for genitives)
    double_remove = Counter()
    doubles = tm_names[1]
    for w in doubles:
        new_token = []
        for token in w:
            # print(token)
            if member(token, gazetteers):
                new_token.append(token)
        new_token = tuple(new_token)
        # print(new_token)
        if new_token != ():
            name_structure = add_name(new_token, name_structure, doubles[w])
            if new_token != w:
                double_remove[w] = doubles[w]
        else:
            double_remove[w] = doubles[w]

    # check triple names (check for genitives)
    triple_names = Counter()
    triple_remove = Counter()
    triples = tm_names[2]
    for w in triples:
        new_token = []
        for token in w:
            # print(token)
            if member(token, gazetteers):
                new_token.append(token)
        new_token = tuple(new_token)
        # print(new_token)
        if new_token != ():
            name_structure = add_name(new_token, name_structure, triples[w])
            if new_token != w:
                triple_remove[w] = triples[w]
        else:
            triple_remove[w] = triples[w]

    # check quadruple names (check for genitives)
    quad_names = Counter()
    quad_remove = Counter()
    quads = tm_names[3]
    for w in quads:
        new_token = []
        for token in w:
            # print(token)
            if member(token, gazetteers):
                new_token.append(token)
        new_token = tuple(new_token)
        # print(new_token)
        if new_token != ():
            name_structure = add_name(new_token, name_structure, quads[w])
            if new_token != w:
                quad_remove[w] = quads[w]
        else:
            quad_remove[w] = quads[w]

    return {'filtered': tuple(name_structure),
            'removed': (single_remove, double_remove, triple_names, quad_names)}



import os
import re

import requests
from IPython.core.display import display
from IPython.display import HTML, Markdown
from PIL import Image

small_scale = 0.59
large_scale = 1.58


def nb_search(
        term='', creator='',
        number=50,
        page=0,
        mediatype='bøker',
        lang="nob",
        period=(18000101, 20401231),
        extra_filters=None,
        extra_conditions=None
):
    """Søk etter term og få ut json"""
    number = min(number, 50)

    filters = []
    aq = []

    if extra_filters is not None:
        filters += extra_filters
    params = {
        'page': page,
        'size': number
    }
    if extra_conditions is not None:
        for x in extra_conditions:
            params[x] = extra_conditions[x]

    if lang != '':
        aq.append(f'languages:{lang}')

    if creator != '':
        filters.append(f'creator:{creator}')

    if mediatype != '':
        filters.append(f'mediatype:{mediatype}')

    if period != ():
        filters.append(f'date:[{period[0]} TO {period[1]}]')

    if filters:
        params['filter'] = filters

    if aq:
        params['aq'] = aq

    if term != '':
        params['q'] = term

    r = requests.get("https://api.nb.no:443/catalog/v1/items", params=params)
    return r.json()


def div_wrapper(div_content):
    return f"""<div>{div_content}</div>"""


def display_books(books, width=100):
    """A dictionary of urns - urls is displayed """

    def html_wrapper(body):
        return f"""<style>
    img {{
        width:{width}px;
        height:auto;
        max-width:100%}}
    </style>
    <body>{body}</body>"""

    book_divs = ""
    for u in books:
        mf = iiif_manifest(u)
        thumbnail = f"<img src = '{mf['thumbnail']['@id']}'></img>"
        metainfo = '\n'.join(
            [f"<b>{x['label']}</b>{x['value']}" for x in mf['metadata']])
        imgs = '\n'.join(
            [f"<img src='{pic_url}'></img>" for pic_url in books[u]])
        book_divs += div_wrapper(thumbnail + metainfo + imgs)
    return html_wrapper(book_divs)


def markdown_books(books, width=100):
    """A dictionary of urns - urls is displayed """

    book_divs = ""
    for u in books:
        mf = iiif_manifest(u)
        thumbnail = f"<h3>Forside</h3> <img src='{mf['thumbnail']['@id']}'></img>\n\n"
        metainfo = '\n'.join([
            f"<h3>Metadata</h3> <b>{x['label']}</b> {x['value']}"
            for x in mf['metadata']
        ])
        imgs = '\n'.join([
            f"<img style='float:right' src='{pic_url}' width = {width}></img>"
            for pic_url in books[u]
        ])
        book_divs += div_wrapper(thumbnail + metainfo + imgs)
    return book_divs


def iiif_manifest(urn):
    if 'URN' not in str(urn) and 'digibok' not in str(urn):
        urn = "URN:NBN:no-nb_digibok_" + str(urn)
    elif 'URN' not in str(urn):
        urn = "URN:NBN:no-nb_" + str(urn)
    r = requests.get(f"https://api.nb.no/catalog/v1/iiif/{urn}/manifest")
    return r.json()


def urns_from_super(
        term='',
        creator='',
        number=50,
        page=0,
        mediatype='bøker',
        lang="nob",
        period=(18000101, 20401231),
        filters=None,
        conditions=None
):
    res = nb_search(term, mediatype=mediatype, creator=creator, number=number,
                    page=page, lang=lang, period=period, extra_filters=filters,
                    extra_conditions=conditions)
    ids = [x['metadata']['identifiers'] for x in res['_embedded']['items']]
    return [x['urn'] for x in ids if 'urn' in x]


def get_illustration_data_from_book(urn):
    if 'digibok' in str(urn):
        urn = re.findall("[0-9]{13}", str(urn))[0]
    r = requests.get('https://api.nb.no/ngram/illustrations', json={'urn': urn})
    return r.json()


def get_urls_from_illustration_data(
        illus, part=True, scale=None, cuts=True, delta=0):
    """
    Restricted images must not go over 1024 x 1024 pixels.

    :param illus: dictionary of this format:

        ``{'height': 270, 'hpos': 251, 'page': 'digibok_2017081626006_0018',
        'resolution': 400, 'vpos': 791, 'width': 373}``

    :param part: If True, return a cut out of the image
    :param scale: To be filled in
    :param cuts: If true, allow cropping of image
    :param delta: To be filled in
    :return: URL of the image
    """
    if scale is None:
        if illus['resolution'] >= 300 or illus['resolution'] < 100:
            scale = large_scale
        else:
            scale = small_scale

    height = int(illus['height']) + 2 * delta
    width = int(illus['width']) + 2 * delta
    vpos = int(illus['vpos']) - delta
    hpos = int(illus['hpos']) - delta

    if cuts:
        if width * scale > 1024:
            width = int(1024 / scale)
        if height * scale > 1024:
            height = int(1024 / scale)

    urn = "URN:NBN:no-nb_" + illus['page']
    url_prefix = "https://www.nb.no/services/image/resolver"
    if part:
        # return cut out
        url = (
            f"{url_prefix}/"
            f"{urn}/"
            f"{int(hpos * scale)},"
            f"{int(vpos * scale)},"
            f"{int(width * scale)},"
            f"{int(height * scale)}"
            f"/full/0/native.jpg"
        )
    else:
        # return whole page
        url = (
            f"{url_prefix}/"
            f"{urn}/"
            f"full/"
            f"0,{part}/"
            f"0/native.jpg"
        )

    return url


def show_illustrations_urn(urn, tilgjengelig='fritt'):
    display(
        Markdown(
            '\n'.join([
                '**' + x['label'] + '**: ' + x['value']
                for x in iiif_manifest(urn)['metadata']
            ])
        )
    )

    if tilgjengelig.lower().startswith('fri'):
        c = False
    else:
        c = True
    return display_finds(
        [
            get_urls_from_illustration_data(u, cuts=c) for u in
            get_illustration_data_from_book(urn)
        ]
    )


def display_finds_meta(r):
    """A list of urls in r is displayed as HTML"""
    rows = [(f"<tr><td><img src='{row}'</td><td><a href = {row} target='_'>"
             f"{row}</a></td></tr>") for row in r]
    return HTML(f"""<html><head></head>
     <body>
     <table>
     {' '.join(rows)}
     </table>
     </body>
     </html>
     """)


def display_finds(r):
    """A list of urls in r is displayed as HTML"""
    rows = [f"<tr><td><img src='{row}'</td></tr>" for row in r]
    return HTML(f"""<html><head></head>
     <body>
     <table>
     {' '.join(rows)}
     </table>
     </body>
     </html>
     """)


def url2urn(url):
    return re.findall("URN:.*[0-9]{13}", url)[0]


def mods(urn):
    r = requests.get(f"https://api.nb.no:443/catalog/v1/metadata/{urn}/mods")
    return r.json()


def pages(urn, scale=800):
    a = iiif_manifest(urn)
    return [page['images'][0]['resource']['@id'].replace(
        "full/full/", f"full/0,{scale}/"
    ) for page in a['sequences'][0]['canvases']]


def load_picture(url):
    r = requests.get(url, stream=True)
    r.raw.decode_content = True
    return r.raw


def get_url(urn, page=1, part=200):
    """From a representaion of URN mapped to digibok.

     A URN is a serial number or any string with serial number.
     """
    urnserial = re.findall('[0-9]+', str(urn))
    if urnserial:
        urnserial = urnserial[0]
    else:
        return ""
    urn = f"URN:NBN:no-nb_digibok_{urnserial}_{page:04d}"
    print(urn)
    url_prefix = "https://www.nb.no/services/image/resolver"
    url = f"{url_prefix}/{urn}/full/0,{part}/0/native.jpg"
    return url


def page_urn(urn, page=1):
    # urn as digit
    return f"URN:NBN:no-nb_digibok_{urn}_{page:04d}"


def super_search(term, number=50, page=0, mediatype='bilder'):
    """Søk etter term og få ut json"""
    number = min(number, 50)
    if term == '':
        r = requests.get(
            "https://api.nb.no:443/catalog/v1/items",
            params={
                'filter': f'mediatype:{mediatype}',
                'page': page,
                'size': number
            }
        )
    else:
        r = requests.get(
            "https://api.nb.no:443/catalog/v1/items",
            params={
                'q': term,
                'filter': f'mediatype:{mediatype}',
                'page': page,
                'size': number
            }
        )
    return r.json()


def find_urls(term, number=50, page=0, mediatype='bilder'):
    """generates urls from super_search for pictures"""
    x = super_search(term, number, page, mediatype=mediatype)
    try:
        urls = [
            f['_links']['thumbnail_custom']['href']
            for f in x['_embedded']['items']
            if f['accessInfo']['accessAllowedFrom'] == 'EVERYWHERE'
               and 'thumbnail_custom' in f['_links']
        ]
    except:  # PEP8: E722 do not use bare 'except', too broad exception clause
        urls = []
    return urls


def find_urls2(term, number=50, page=0):
    """generates urls from super_search for pictures"""
    x = super_search(term, number, page)
    try:
        urls = [
            f['_links']['thumbnail_custom']['href']
            for f in
            x['_embedded']['mediaTypeResults'][0]['result']['_embedded'][
                'items']
            if f['accessInfo']['accessAllowedFrom'] == 'EVERYWHERE'
               and 'thumbnail_custom' in f['_links']
        ]
    except:  # PEP8: E722 do not use bare 'except', too broad exception clause
        urls = [' ... hmm ...']
    return urls


def get_picture_from_urn(urn, width=0, height=300):
    meta = iiif_manifest(urn)
    url_prefix = "https://www.nb.no/services/image/resolver"
    if 'error' not in meta:
        if width == 0 and height == 0:
            url = f"{url_prefix}/{urn}/full/full/0/native.jpg"
        else:
            url = f"{url_prefix}/{urn}/full/{width},{height}/0/native.jpg"
        # print(url)
    return Image.open(load_picture(url))


def get_picture_from_url(url, width=0, height=300):
    return Image.open(
        load_picture(
            url.format(width=width, height=height)
        )
    )


def get_metadata_from_url(url):
    urn = re.findall("(URN.*?)(?:/)", url)[0]
    triple = iiif_manifest(urn)
    # print(urn, triple)
    r = {}  # Local variable 'r' value is not used
    if 'error' not in triple:
        r = {x['label']: x['value'] for x in triple['metadata'] if 'label' in x}
    else:
        r = triple['error']
    return r


def find_urns(term):
    """From result of super_search, to be fed into iiif_manifest."""

    ss = super_search(term)
    urns = [
        f['metadata']['identifiers']['urn']
        for f in
        ss['_embedded']['mediaTypeResults'][0]['result']['_embedded']['items']
        if 'urn' in f['metadata']['identifiers']
    ]
    return urns


def total_search(size=50, page=0):
    """Finn de første antallet = 'size' fra side 'page' og få ut json"""
    size = min(size, 50)
    r = requests.get(
        "https://api.nb.no:443/catalog/v1/items",
        params={
            'filter': 'mediatype:bilder',
            'page': page,
            'size': size
        }
    )
    return r.json()


def total_urls(number=50, page=0):
    """find urls sequentially"""
    x = total_search(number, page)
    try:
        urls = [
            f['_links']['thumbnail_custom']['href']
            for f in x['_embedded']['items']
            if f['accessInfo']['accessAllowedFrom'] == 'EVERYWHERE'
               and 'thumbnail_custom' in f['_links']
        ]
    except:  # PEP8: E722 do not use bare 'except', too broad exception clause
        urls = []
    return urls


def find_urns_sesam(term='', creator='', number=50, page=0, mediatype='bilder'):
    """generates urls from super_search for pictures"""
    x = nb_search(term=term, creator=creator, number=number, page=page,
                  mediatype=mediatype)
    try:
        sesamid = [
            f['id']
            for f in x['_embedded']['items']
            if f['accessInfo']['accessAllowedFrom'] == 'EVERYWHERE'
               and 'thumbnail_custom' in f['_links']
        ]
    except:
        sesamid = []
    return sesamid


def save_pictures(pages_, urn, root='.'):
    """Save picture references in pages on the form::

        pages = {
            urn1: [page1, page2, ..., pageN],
            urn2: [page1, ..., pageM],
            ...
            urnK: [page1, ..., pageL]}

    Parameter urn is one of the keys in pages,
    where each page reference is a URL.
    """

    # In case urn is an actual URN, works also if urn is passed as sesamid

    folder_name = urn.split(':')[-1]
    folder_ref = os.path.join(root, folder_name)
    try:
        os.mkdir(folder_ref)

    except FileExistsError:
        pass

    for p in pages_[urn]:
        # pell ut entydig referanse til bildet fra URL-en i bildelisten som filnavn

        filename = p.split('/')[6].split(':')[-1] + '.jpg'

        path = os.path.join(folder_ref, filename)
        get_picture_from_url(p).save(path)

    return True


def save_all_pages(pages_, root='.'):
    """Save picture references in pages on the form::

        pages = {
            urn1 : [page1, page2, ..., pageN],
            urn2: [page1, ..., pageM],
            ...
            urnK: [page1, ..., pageL]
        }

    Each page reference is a URL.
    """

    # In case urn is an actual URN, works also if urn is passed as sesamid
    for urn in pages_:
        folder_name = urn.split(':')[-1]
        folder_ref = os.path.join(root, folder_name)
        try:
            os.mkdir(folder_ref)

        except FileExistsError:
            pass

        for p in pages_[urn]:
            # pell ut entydig referanse til bildet fra URL-en i bildelisten som filnavn

            filename = p.split('/')[6].split(':')[-1] + '.jpg'

            path = os.path.join(folder_ref, filename)
            get_picture_from_url(p).save(path)

    return True


def json2html(meta):
    items = [f"<dt>{key}</dt><dd>{value}</dd>" for key, value in meta.items()]
    result = f"<dl>{' '.join(items)}</dl>"
    return result



#!/usr/bin/ python3
# -*- coding: utf-8 -*-

"""
Tokenisator for ngramleser (evt. parsing).
-------------------------------------------
Lars G Johnsen, Nasjonalbiblioteket, juni 2014

Tokenisatorens oppgave er å danne token eller ord fra en sekvens med tegn.
I utgangspunktet fungerer skilletegn og mellomrom som ordgrenser,
men det er unntak, se listen nedenfor. Skilletegn danner som oftest egne token,
men spesielt punktum og komma brukes på flere måter, noe det må tas høyde for.

Noen ord (token) har bestanddeler i form av skilletegn, som forkortelser, tall,
i tillegg kan ordene selv være bundet sammen med bindestrek:

p-pille (bindestrek)
3.3 (punktum i seksjonsnummerering)
etc. (forkortelser)
10 000 (token over mellomrom)
3,14 (desimaltall med komma)
co2 (bokstaver og tall i kjemiske formler)
co2-forurensning (bokstaver tall pluss bindestrek)
17. (ordenstall som i 17. mai)
P. A. Munch (punktum i initialer)
... tre eller flere  punktum
Når punktum følger tall vil tokenisatoren la punktum tilhøre tallet
med mindre punktumet følges av mellomrom og stor bokstav.

Punktum tilhører alle forkortelser som tar punktum uavhenging av kontekst.
Den kan imidlertid gjøres følsom for påfølgende stor bokstav,
men det er altså ikke gjort her.
Punktum tillates inne i ord og deler ikke opp ord med punktum i seg.

Alle skilletegn ellers utgjør egne token, bortsett fra § som kan sekvensieres,
så § og §§ vil være egne tokener;
de benyttes en hel del i lovtekster for entall og flertall.

Tall skrevet med mellomrom blir ett token om de er på formen xx xxx, altså 1
eller 3 siffer etterfulgt av grupper på tre siffer skilt med ett mellomrom.
Så 3 1995 vil være to tokener, mens 3 995 blir ett token,
4000 398 blir igjen to token. (Mulig det er endret)

Tall som følger etter § (adskilt med maks ett mellomrom)
vil aldri tiltrekke seg punktum.

Øvrige tegn som ikke passer inn med mønstrene over behandles som separate token.

================================================================================
"""

import re
import sys
import time

"""Her er listen over forkortelser med punktum.

Hentet fra Wikipedia og Språkrådet med egendefinerte tillegg.
"""
fork = [
    'L\.',
    'A/S',
    'a/s',
    'R[.]?I[.]?P.',
    '[aA]b\.prov\.',
    '[aA]plf\.',
    '[eE]\.[kK]r\.',
    '[Aa][.]?D\.',
    'a[.]?l\.',
    'a[.]?m\.',
    'adm\.',
    'adr\.',
    'adv\.',
    'alt\.',
    'ank\.',
    'apr\.',
    'aq\.',
    'ass\.',
    'att\.',
    'aug\.',
    'aut\.',
    'avd\.',
    'avg\.',
    'avst\.',
    'b\.c\.',
    'bd\.',
    'bed\.øk\.',
    'bet\.',
    'bill\.mrk\.?',
    'bill\.',
    'bl[.]?a\.',
    'bm\.',
    'bnr\.',
    'bto\.',
    'bvl\.',
    'c[.]?c\.',
    'ca\.',
    'cand\.',
    'd[.]?d\.',
    'd[.]?e\.',
    'd[.]?m\.',
    'd[.]?y\.',
    'd[.]?y\.',
    'd\.å\.',
    'd\.s\.',
    'd\.'
    'd\.e\.',
    'dept\.',
    'des\.',
    'dfm\.',
    'dg\.',
    'dir\.',
    'disp\.',
    'div\.',
    '[Dd]r\.',
    'ds\.',
    'dss\.',
    'dvs\.',
    'e\.f\.',
    'e\.g\.',
    'e\.kr\.',
    'e\.l\.',
    'e\.g\.',
    'eg\.',
    'eks\.',
    'ekskl\.',
    'eksp\.',
    'et\.',
    'etab\.',
    'etabl\.',
    'etc\.',
    'ev\.',
    'evt\.',
    'f[.]?eks\.',
    'f\.kr\.',
    'f[.]?m\.',
    'f[.]?o[.]?f\.',
    'f\.o\.m[.]',
    'f\.t\.',
    'f\.å\.',
    'f\.[Kk]r\.',
    'f\.ø\.',
    'feb\.',
    'fhv\. ',
    'fig\.',
    'fl\.',
    'forf\.',
    'fork\.',
    'forts\.',
    'fr\.',
    'framh\.',
    'fre\.',
    '[Ff]rk\.',
    'fung\.',
    'fv\.',
    'fvt\.',
    'g\.',
    'g\.; g[.]?m\.',
    'gl\.',
    'gnr\.',
    'gt\.',
    '[Hh]r\.?',
    'h[.]?r\.adv\.',
    'h\.v\.',
    'hhv\.',
    'hoh\.',
    'hå\.',
    'i[.]?e\.',
    'ib\.'
    'ibid\.',
    'ifb\.',
    'ifbm\.',
    'ift\.',
    'iht\.',
    'ill\.',
    'ing\.',
    'inkl\.',
    'innb\.',
    'inst\.',
    'istf\.',
    'jan\.',
    'jf\.',
    'jfr\.',
    'jnr\.',
    'jr\.',
    'jun\.',
    'jur\.',
    'kap\.',
    'kfr\.',
    'kgl[.]?res\.',
    'kgl\.res\.',
    'kl\.',
    'komm\.',
    'kst\.',
    'kto\.',
    'kv\.',
    'l[.]?c\.',
    'lab\.',
    'lau\.',
    'lib\.',
    'lnr\.',
    'loc\.cit\.',
    'lt\.',
    'lø\.',
    'lør\.',
    'm[.]?a[.]?o\.',
    'm[.]?a\.',
    'm[.]?m\.',
    'm\.o\.t\.?',
    'm\.m[.]?',
    'mm[.]',
    'ma\.',
    'mag[.]?art\.',
    'mag\.',
    'maks\.',
    'man\.',
    'mao\.',
    'max\.',
    'md\.',
    'med\.',
    'mek\.',
    'mfl\.',
    'mht\.',
    'mil\.',
    'mill\.',
    'min\.',
    'mnd\.',
    'mod\.',
    'moh\.',
    'mrd\.',
    'ms\.',
    'muh\.',
    'mv\.',
    'm.\s*v\.',
    'mva\.',
    'n[.]?br\.',
    'n\.å\.',
    'ndf\.',
    'no\.',
    'nov\.',
    'nr\.',
    'nto\.',
    'nyno\.',
    'o[.]?a\.',
    'o\.a\.',
    'o\.l\.',
    'obs\.',
    'off\.',
    'ofl\.',
    'okt\.',
    'on\.',
    'ons\.',
    'op\.',
    'osb\.',
    'osv\.',
    'ot\.prp\.',
    'ovf\.',
    'p\.a\.',
    'p[.]?m\.',
    'p[.]?p\.',
    'p[.]?t\.',
    'p\.',
    'p\.t[.]',
    'pb\.',
    'pga\.',
    'ph[.]?d\.',
    'pharm\.',
    'philol\.',
    'pkt\.',
    'pr\.',
    'pst\.',
    'r\.',
    'red\.',
    'ref\.',
    'reg\.',
    'rek\.',
    'rv\.',
    's[.]?d\.',
    's\.k\.',
    's[.]?m\.',
    's[.]?u\.',
    's\.å\.',
    'sek\.',
    'sen\.sep\.',
    'sep\.',
    'sign\.',
    'siv[.]?ing\.',
    'siv\.ing\.',
    'sj\.',
    'sos\.',
    'sos\.øk\.',
    'sp\.',
    'spm\.',
    'sr\.',
    'sst\.',
    'st[.]?meld\.',
    'st[.]?prp\.',
    'st\.',
    'stk\.',
    'stp\.',
    'str\.',
    'strpl\.',
    'stud\.',
    'stv\.',
    'su\.',
    'såk\.',
    'sø\.',
    'søn\.',
    't[.]?d\.',
    't\.o\.m[.]?',
    'tab\.',
    'techn\.',
    'temp\.',
    'ti\.',
    'tidl\.',
    'tils\.',
    'tilsv\.',
    'tlf\.',
    'to\.',
    'ty\.',
    'tvml\.',
    'ult\.',
    'utg\.',
    'v\.',
    'vedk\.',
    'vedr\.',
    'vg\.',
    'vgs\.',
    'vha\.',
    'vit\.ass\.',
    'vn\.',
    'vol\.',
    'vs\.',
    'vsa\.',
    'årg\.',
    'årh\.'
]

"""-----------------------------------------------------------------------------

numeriske uttrykk er alt som er bygd opp av tall, komma og punktum og blanke.

LGJ: juni 2014

--------------------------------------------------------------------------------
"""

"""Tall som kan slutte på punktum består av hele tall, som tokeniseres
 med punktum bare om neste påfølgende tegn (etter blank) er stor bokstav.

Denne må justers for samisk og andre språk med større utvalg av store bokstaver.
Det vil sannsynligvis ikke ha så veldig stor betydning for utfallet.
"""
num = r'\d+(?:\.(?! [A-ZÆØÅ]))?'

"""
f.eks. 10 000, tillater ikke punktum. Tokeniserer tall med mellomrom der de forekommer.

"""

num0 = r'\d{1,3}(?:\s\d\d\d(?!\d))+'

"""seksjon 3.2.1 eller 2.3999, kan ikke ha sluttpunktum."""
num1 = r'\d+(?:\.\d+)+'

"""3,5 kan ikke ha sluttpunktum."""
num2 = r'\d+,\d+'

"""Det var .2 prosent økning."""
num3 = r'\.\d+'

"""Tre eller flere punktum blir ett token."""
num4 = r'\.\.\.+'

# kombiner til det regulære uttrykket num eller num0 eller...
num = '|'.join([num0, num1, num2, num3, num4, num])

"""Paragraftegn kan komme i en eller to (eller flere?) utgaver."""
# tolk tall etter § som rene tall uten punktum,
# men også med bindestrek så i § 2-5  blir 2-5 et token.
parnum0 = r'(?<=§ )\d+(?:[-–—]\d+)*|(?<=§)\d+(?:[-–—]\d+)*'

parnum = r'(?<=§ )\d+|(?<=§)\d+'  # tolk tall etter § som heltall uten punktum.
paragrafer = '§+'  # § eller §§ ukes i lovtekster

"""Ord er alt som ikke innholder skillende skilletegn.

Bindestrek og @ og lignende går inn i tokenet, punktum inkludert,
tar også med @ for mailadresser.
Bindestrek kan også avslutte ord som i "ord- og setningsdeling".
Andre tegn, som punktum og kolon i slutt, vil ikke tokeniseres sammen med ordet.
"""

initialer = r'(?<=(?: |\.))[A-ZÆØÅ]\.'
word = r'\w+[-\d.@]*[\w\d]+-?'

word = '|'.join([initialer, word])

"""Alle tegn som ikke er et blankt tegn
 (tab, mellomrom linjeskift osv.),
 og som ikke er blitt matchet opp tidligere,
 blir å regne som egne token.
"""

catchall = r'\S'  # alle tegn som ikke er blanke blir til et eget token

"""-----------------------------------------------------------------------------

Kombiner alle uttrykkene i rekkefølge og kompiler dem.
Sjekk først om det er en forkortelse, ellers sjekk om det er et tall,
sjekk paragrafer, prøv å lag et ord. Hvis ikke noe av det her,
la tegnet være sitt eget token og gå videre.

--------------------------------------------------------------------------------
"""

regex = fork + [parnum0, parnum, num, paragrafer, word, catchall]
regex = re.compile('|'.join(regex))


def tokenize_timer(text):
    t0 = time.process_time()
    tokens = re.findall(regex, text)
    t1 = time.process_time()
    t = t1 - t0
    print(f"tid: {t}")
    return tokens


def tokenize(text):
    tokens = re.findall(regex, text)
    return tokens


if __name__ == "__main__":
    try:
        with open(sys.argv[1], "r", encoding="utf-8") as charfile:
            for token in tokenize(charfile.read()):
                print(token, '\n')
    except:
        print("Får ikke åpnet fila \"%s\"" % (sys.argv[1],))



from datetime import datetime

from ..api.dhlab_api import ngram_book, ngram_news
from .nb_ngram import nb_ngram

class Ngram():
    """Top level class for ngrams"""
    def __init__(self, words = None, from_year = None, to_year = None, doctype = None, lang = 'nob'):
       
        self.date = datetime.now()
        if to_year is None:
            to_year = self.date.year
        if from_year is None:
            from_year = 1950
            
        self.from_year = from_year
        self.to_year = to_year
        self.words = words
        self.lang = lang
        if not doctype is None:
            if 'bok' in doctype:
                doctype = 'bok'
            elif 'avis' in doctype:
                doctype = 'avis'
            else:
                doctype = 'bok'
        else:
            doctype = 'bok'
        ngrm = nb_ngram(terms = ', '.join(words), corpus = doctype, years = (from_year, to_year))
        ngrm.index = ngrm.index.astype(str)
        self.ngram = ngrm
        return None

    def plot(self, **kwargs):
        self.ngram.plot(**kwargs)
    
    def compare(self, another_ngram):
        from datetime import datetime
        start_year = max(datetime(self.from_year,1,1), datetime(another_ngram.from_year,1,1)).year
        end_year = min(datetime(self.to_year,1,1), datetime(another_ngram.to_year,1,1)).year
        compare =  (self.ngram.loc[str(start_year):str(end_year)].transpose()/another_ngram.ngram[str(start_year):str(end_year)].transpose().sum()).transpose()
        return compare

    
class Ngram_book(Ngram):
    """Extract ngrams using metadata with functions to be inherited"""

    def __init__(
        self,
        words = None,
        title = None,
        publisher = None,
        city = None,
        lang = 'nob',
        from_year = None,
        to_year = None,
        ddk = None,
        subject = None
    ):

        self.date = datetime.now()
        if to_year is None:
            to_year = self.date.year
        if from_year is None:
            from_year = 1950
        self.from_year = from_year
        self.to_year = to_year
        self.words = words
        self.title = title
        self.publisher = publisher
        self.city = city
        self.lang = lang
        self.ddk = ddk
        self.subject = subject
        self.ngram = ngram_book(word = words, title = title, publisher = publisher, lang = lang,city = city, period = (from_year, to_year), ddk = ddk, topic = subject)
        #self.cohort =  (self.ngram.transpose()/self.ngram.transpose().sum()).transpose()
        return None
    
class Ngram_news(Ngram):
    def __init__(
        self, 
        words = None,
        title = None,
        city = None,
        from_year = None,
        to_year = None
    ):
        self.date = datetime.now()
        if to_year is None:
            to_year = self.date.year
        if from_year is None:
            from_year = 1950
        self.from_year = from_year
        self.to_year = to_year
        self.words = words
        self.title = title
        self.ngram = ngram_news(word = words, title = title, period = (from_year, to_year))
        #self.cohort =  (self.ngram.transpose()/self.ngram.transpose().sum()).transpose()
        return None



import pandas as pd
from ..api.nb_ngram_api import get_ngram

def nb_ngram(terms, corpus='bok', smooth=3, years=(1810, 2010), mode='relative'):
    df = ngram_conv(get_ngram(terms, corpus=corpus), smooth=smooth, years=years, mode=mode)
    df.index = df.index.astype(int)
    return df.sort_index()

def ngram_conv(ngrams, smooth=1, years=(1810,2013), mode='relative'):
    ngc = {}
    # check if relative frequency or absolute frequency is in question
    if mode.startswith('rel') or mode=='y':
        arg = 'y'
    else:
        arg = 'f'
    for x in ngrams:
        if x != []:
            ngc[x['key']] = {
                z['x']:z[arg]
                for z in x['values'] 
                if int(z['x']) <= years[1] and int(z['x']) >= years[0]
            }
    return pd.DataFrame(ngc).rolling(window=smooth, win_type='triang').mean()





import requests
import pandas as pd
import json


BASE_URL = "https://api.nb.no/dhlab"

def get_metadata(urns = None):
    """ Fetch metadata from a list of urns """
    params = locals()
    r = requests.post(f"{BASE_URL}/get_metadata", json = params)
    return pd.DataFrame(r.json())



import requests
import pandas as pd
import json
import re

BASE_URL = "https://api.nb.no/dhlab"

pd.options.display.max_rows = 100

# fetch metadata

def get_metadata(urns = None):
    """ Fetch metadata from a list of urns """
    params = locals()
    r = requests.post(f"{BASE_URL}/get_metadata", json = params)
    return pd.DataFrame(r.json())

def get_reference(
    corpus = 'digavis',
    from_year = 1950,
    to_year = 1955,
    lang = 'nob',
    limit = 100000
):
    params = locals()
    r = requests.get(BASE_URL + "/reference_corpus", params = params)
    if r.status_code == 200:
        result = r.json()
    else:
        result = []
    return pd.DataFrame(result, columns = ['word', 'freq']).set_index('word')

def find_urns(docids = None, mode = 'json'):
    """ Return a list of URNs from a list of docids as
    a dictionary {docid: URN} or as a pandas dataframe"""

    params = locals()
    r = requests.post(BASE_URL + "/find_urn", json = params)
    if r.status_code == 200:
        res = pd.DataFrame.from_dict(r.json(), orient = 'index', columns = ['urn'])
    else:
        res = pd.DataFrame()
    return res

def ngram_book(
    word = ['.'], 
    title = None, 
    period = None, 
    publisher = None, 
    lang=None, 
    city = None, 
    ddk = None, 
    topic = None
):
    """Get a time series for a word as string, 
    title is name of book period is (year, year),
    lang is three letter iso code.
    Use % as wildcard where appropriate - no wildcards in word and lang"""
    params = locals()
    if isinstance(word, str):
        # assume a comma separated string
        word = [w.strip() for w in word.split(',')]
    params['word'] = tuple(word)
    params = {x:params[x] for x in params if not params[x] is None}
    r = requests.post(BASE_URL + "/ngram_book", json = params)
    #print(r.status_code)
    df = pd.DataFrame.from_dict(r.json(), orient = 'index')
    df.index = df.index.map(lambda x: tuple(x.split()))
    columns = df.index.levels[0]
    df = pd.concat([df.loc[x] for x in columns], axis = 1)
    df.columns = columns 
    #df.index = df.index.map(pd.Timestamp)
    return df

def ngram_periodicals(
    word = ['.'], 
    title = None,
    period = None,
    publisher = None,
    lang=None,
    city = None,
    ddk = None,
    topic = None
):
    """Get a time series for a word as string,
    title is name of periodical period is (year, year),
    lang is three letter iso code.
    Use % as wildcard where appropriate - no wildcards in word and lang"""
    
    params = locals()
    if isinstance(word, str):
        # assume a comma separated string
        word = [w.strip() for w in word.split(',')]
    params['word'] = tuple(word)
    params = {x:params[x] for x in params if not params[x] is None}
    r = requests.post(BASE_URL + "/ngram_periodicals", json = params)
    #print(r.status_code)
    df = pd.DataFrame.from_dict(r.json(), orient = 'index')
    df.index = df.index.map(lambda x: tuple(x.split()))
    columns = df.index.levels[0]
    df = pd.concat([df.loc[x] for x in columns], axis = 1)
    df.columns = columns 
    #df.index = df.index.map(pd.Timestamp)
    return df


def ngram_news(word = ['.'], title = None, period = None):
    """ get a time series period is a tuple of (year, year), (yearmonthday, yearmonthday) 
    word is string and title is the title of newspaper, use % as wildcard"""
    params = locals()
    if isinstance(word, str):
        # assume a comma separated string
        word = [w.strip() for w in word.split(',')]
    params['word'] = tuple(word)
    params = {x:params[x] for x in params if not params[x] is None}
    r = requests.post(BASE_URL + "/ngram_newspapers", json = params)
    #print(r.status_code)
    df = pd.DataFrame.from_dict(r.json(), orient = 'index')
    df.index = df.index.map(lambda x: tuple(x.split()))
    columns = df.index.levels[0]
    df = pd.concat([df.loc[x] for x in columns], axis = 1)
    df.columns = columns
    #df.index = df.index.map(pd.Timestamp)
    return df

def get_document_frequencies(urns = None, cutoff = 0, words = None):
    params = locals()
    r = requests.post(BASE_URL + "/frequencies", json = params)
    result = r.json()
    structure = {u[0][0] : dict([tuple(x[1:3]) for x in u]) for u in result if u != []}
    df = pd.DataFrame(structure)
    return df.sort_values(by = df.columns[0], ascending = False)

def get_word_frequencies(urns = None, cutoff = 0, words = None):
    params = locals()
    r = requests.post(BASE_URL + "/frequencies", json = params)
    result = r.json()
    structure = {u[0][0] : dict([(x[1],x[2]/x[3]) for x in u]) for u in result if u != []}
    df = pd.DataFrame(structure)
    return df.sort_values(by = df.columns[0], ascending = False)


def get_document_corpus(**kwargs):
    return document_corpus(**kwargs)

def document_corpus(
    doctype = None,
    author = None,
    freetext = None,
    from_year = None,
    to_year = None,
    from_timestamp = None,
    to_timestamp = None,
    title = None,
    ddk = None,
    subject = None,
    lang = None,
    limit = None
):
    """ Fetch a corpus based on metadata - doctypes are digibok, digavis, digitidsskrift"""
    
    parms = locals()
    params = {x:parms[x] for x in parms if not parms[x] is None }
    if "ddk" in params:
        params["ddk"]  = "^" + params['ddk'].replace('.', '"."')
        
    r = requests.post(BASE_URL + "/build_corpus", json=params)
    
    return pd.DataFrame(r.json())
    
def urn_collocation(
    urns = None,
    word = 'arbeid',
    before = 5,
    after = 0,
    samplesize = 200000
):
    """ Create a collocation from a list of URNs - 
    returns distance (sum of distances and bayesian distance) and frequency"""
    
    params = {
        'urn': urns,
        'word': word,
        'before': before,
        'after': after,
        'samplesize': samplesize
    }
    r = requests.post(BASE_URL + "/urncolldist_urn", json = params)
    return pd.read_json(r.text)

def totals(n = 50000):
    """ Get total frequencies of words in database"""
    
    r = requests.get(BASE_URL + "/totals/{n}".format(n = n))
    return pd.DataFrame.from_dict(dict(r.json()),orient = 'index', columns = ['freq'])
    
def concordance(urns = None, words = None, window = 25, limit = 100):
    """ Get a list of concordances from database, words is an fts5 string search expression"""
    if words is None:
        return {} # exit condition
    else:
        params = {
            'urns': urns,
            'query': words,
            'window': window,
            'limit': limit
        }
        r = requests.post(BASE_URL + "/conc", json = params)
    return pd.DataFrame(r.json())

def concordance_counts(urns = None, words = None, window = 25, limit = 100):
    """ Get a list of concordances from database, words is an fts5 string search expression"""
    if words is None:
        return {} #exit condition
    else:
        params = {
            'urns': urns,
            'query': words,
            'window': window,
            'limit': limit
        }
        r = requests.post(BASE_URL + "/conccount", json = params)
    return pd.DataFrame(r.json())

def konkordans(urns = None, query = None, window = 25, limit = 100):
    if query is None:
        return {} # exit condition
    else:
        params = {
            'urns': urns,
            'query': query,
            'window': window,
            'limit': limit
        }
        r = requests.post(BASE_URL + "/conc", json = params)
    return pd.DataFrame(r.json())

def collocation(corpusquery = 'norge', word = 'arbeid', before = 5, after = 0):
    params = {
        'metadata_query': corpusquery,
        'word': word,
        'before': before,
        'after': after
    }
    r = requests.post(BASE_URL + "/urncolldist", json = params)
    return pd.read_json(r.text)



import json

import requests
import networkx as nx

NGRAM_API = "https://api.nb.no/dhlab/nb_ngram/ngram/query"
GALAXY_API = "https://api.nb.no/dhlab/nb_ngram_galaxies/galaxies/query"

def get_ngram(terms, corpus='avis'):
    req = requests.get(
        NGRAM_API,
        params = {
            'terms':terms,
            'corpus':corpus
        }
    )
    if req.status_code == 200:
        res = req.text
    else:
        res = "[]"
    return json.loads(res)


def make_word_graph(words, corpus = 'all', cutoff = 16, leaves = 0):
    """Get galaxy from ngram-database.
    corpus is bok, avis or both
    words is a commaseparated string
    English and German provided by Google N-gram.
    Set leaves=1 to get the leaves. Parameter cutoff only works for lang='nob'.
    Specify English by setting lang='eng' and German by lang='ger'"""
    
    params = dict()
    params['terms'] = words
    params['corpus'] = corpus
    params['limit'] = cutoff
    params['leaves'] = leaves
    result = requests.get(GALAXY_API, params=params)
    G = nx.DiGraph()
    edgelist = []
    if result.status_code == 200:
        graph = json.loads(result.text)
        #print(graph)
        nodes = graph['nodes']
        edges = graph['links']
        for edge in edges:
            edgelist += [
                (nodes[edge['source']]['name'],
                 nodes[edge['target']]['name'],
                 abs(edge['value']))
            ]
    #print(edgelist)
    G.add_weighted_edges_from(edgelist)
    return G



import re
from collections import Counter
from corpus import Corpus
from pandas import DataFrame

import pandas as pd

from ..api.dhlab_api import concordance, urn_collocation

# convert cell to a link
def make_link(row):
    r = "<a target='_blank' href = 'https://urn.nb.no/{x}'>{x}</a>".format(x = str(row))
    return r

# find hits a cell
find_hits = lambda x: ' '.join(re.findall("<b>(.+?)</b", x))



def urnlist(corpus):
    """Try to pull out a list of URNs from corpus"""
    
    if isinstance(corpus, Corpus):
        urnlist = list(corpus.corpus.urn)
    elif isinstance(corpus, DataFrame):
        urnlist = list(corpus.urn)
    else:
        urnlist = []
    return urnlist


class Concordance:
    """Wrapper for concordance function with added functionality"""
    def __init__(self, corpus, query):

            
        self.concordance = concordance(urns = urnlist(corpus), words = query)
        self.concordance['link'] = self.concordance.urn.apply(make_link)
        self.concordance = self.concordance[['link', 'urn', 'conc']]
        self.concordance.columns = ['link', 'urn', 'concordance']
        self.corpus = corpus
        self.size = len(self.concordance)
    
    def show(self, n = 10, style = True):
        if style:
            result =  self.concordance.sample(min(n, self.size))[['link', 'concordance']].style
        else:
            result =  self.concordance.sample(min(n, self.size))
        return result
    
class Collocations():
    """Collocations """
    def __init__(
        self,
        corpus = None,
        words = None,
        before = 10,
        after = 10,
        reference = None
    ):
        if isinstance(words, str):
            words = [words]
        coll = pd.concat(
            [
                urn_collocation(
                    urns = urnlist(corpus),
                    word = w,
                    before = before,
                    after = after
                ) 
                for w in words
            ]
        )[['counts']]

        self.coll = coll.groupby(coll.index).sum()
        self.reference = reference
        self.before = before
        self.after = after
        
        if reference is not None:
            teller = self.coll.counts/self.coll.counts.sum()
            divisor = self.reference.freq/self.reference.freq.sum()
            self.coll['relevance'] = teller/divisor
    
    def show(self, sortby = 'counts', n = 20):
        return self.coll.sort_values(by = sortby, ascending = False)
    
    def keywordlist(self, top = 200, counts = 5, relevance = 10):
        mask = self.coll[self.coll.counts > counts]
        mask = mask[mask.relevance > relevance]
        return list(mask.sort_values(by = 'counts', ascending = False).head(200).index)



import json

import requests
import pandas as pd

def load_picture(url):
    r = requests.get(url, stream=True)
    r.raw.decode_content=True
    #print(r.status_code)
    return r.raw

def iiif_manifest(urn):
    r = requests.get(f"https://api.nb.no/catalog/v1/iiif/{urn}/manifest")
    return r.json()

def mods(urn):
    r = requests.get(f"https://api.nb.no:443/catalog/v1/metadata/{urn}/mods")
    return r.json()

def super_search(term, number=50, page=0, mediatype='bilder'):
    """Søk etter term og få ut json"""
    number = min(number, 50)
    if term == '':
        r = requests.get(
            "https://api.nb.no:443/catalog/v1/items", 
             params = { 
                 'filter':f'mediatype:{mediatype}', 
                 'page':page, 
                 'size':number
             }
        )
    else:        
        r = requests.get(
            "https://api.nb.no:443/catalog/v1/items", 
             params = {
                 'q':term, 
                 'filter':f'mediatype:{mediatype}', 
                 'page':page, 
                 'size':number
             }
        )
    return r.json()

def total_search(size=50, page=0):
    """Finn de første antallet = 'size' fra side 'page' og få ut json"""
    size = min(size, 50)
    r = requests.get(
        "https://api.nb.no:443/catalog/v1/items", 
         params = {
             'filter':'mediatype:bilder', 
             'page':page, 
             'size':size
         }
    )
    return r.json()


def get_df(frases, title='aftenposten'):
    """get document phrases"""
    querystring = " + ".join(['"'+frase+'"' for frase in frases])
    query = {
        'q':querystring,
        'size':1,
        'aggs':'year',
        #'filter':'mediatype:{mt}'.format(mt=media),
        'filter':f'title:{title}'
    }
    r = requests.get("https://api.nb.no/catalog/v1/items", params = query)
    aggs = r.json()['_embedded']['aggregations'][0]['buckets']
    return {x['key']:x['count'] for x in aggs}

def get_json(frases, mediatype='aviser'):

    querystring = " + ".join(['"'+frase+'"' for frase in frases])
    query = {
        'q':querystring,
        'size':1,
        'snippets':mediatype,
        'aggs':'year',
        
#        'filter':'mediatype:{mt}'.format(mt=mediatype),
        'searchType':'FULL_TEXT_SEARCH'
        #'filter':'title:{title}'.format(title=title)
    }
    r = requests.get("https://api.nb.no/catalog/v1/items", params = query)
    aggs = r.json()
    return aggs

def get_data(frase, media='avis', title='jazznytt'):

    query = {
        'q':'"'+frase+'""',
        'size':1,
        'aggs':'year',
        'filter':'mediatype:{mt}'.format(mt=media),
        'filter':'title:{title}'.format(title=title)
    }
    r = requests.get("https://api.nb.no/catalog/v1/items", params = query)
    return r.json()

def get_data_and(frases, title='aftenposten', media='avis'):

    querystring = " + ".join(['"'+frase+'"' for frase in frases])
    print(querystring)
    query = {
        'q':querystring,
        'size':1,
        'aggs':'year',
        #'filter':'mediatype:{mt}'.format(mt=media),
        'filter':'title:{title}'.format(title=title)
    }
    r = requests.get("https://api.nb.no/catalog/v1/items", params = query)
    return r.json()

def get_df_pd(frase, media='bøker'):
    return pd.DataFrame.from_dict(get_df(frase, media=media ), orient='index').sort_index()


def get_konks(urn, phrase, window=1000, n = 1000):

    querystring = '"'+ phrase +'"' 
    query = {
        'q':querystring,
        'fragments': n,
        'fragSize':window
       
    }
    r = requests.get("https://api.nb.no/catalog/v1/items/{urn}/contentfragments".format(urn=urn), params = query)
    res = r.json()
    results = []
    try:
        for x in res['contentFragments']:
            urn = x['pageid']
            hit = x['text']
            splits = hit.split('<em>')
            s2 = splits[1].split('</em>')
            before = splits[0]
            word = s2[0]
            after = s2[1]
            results.append({'urn': urn, 'before': before, 'word':word, 'after':after})
    except:
        True
    return results

def get_phrase_info(urn, phrase, window=1000, n = 1000):

    querystring = '"'+ phrase +'"' 
    query = {
        'q':querystring,
       
    }
    r = requests.get(f"https://api.nb.no/catalog/v1/items/{urn}/contentfragments", params = query)
    res = r.json()
    return res



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dhlab",
    version="2.0.3",
    author="The National Library of Norway",
    author_email="dh-lab@nb.no",
    description="Library for text and image analysis by the Digital Humanities lab (DH-lab)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://dhlab.readthedocs.io",
    project_urls={
        "GitHub": "https://github.com/NationalLibraryOfNorway/DHLAB",
        "Bug Tracker": "https://github.com/NationalLibraryOfNorway/DHLAB/issues",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.2',
    install_requires=[
        'beautifulsoup4',
        'ipython',
        'ipywidgets',
        'matplotlib',
        'networkx',
        'numpy',
        'pandas',
        'Pillow',
        'python_louvain',
        'requests',
        'seaborn',
        'setuptools',
	'wordcloud'
    ]
)
